/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <malloc.h>



#define VER    "0.2.2"
#define PK1    0x04034b50    /* local_head */
#define PK2    0x02014b50    /* central_dir */
#define PK3    0x06054b50    /* end_central_dir */
#define PK4    0x05054b50    /* digital sign */
#define PK5    0x06064b50    /* ZIP64 */



void zipsize1(void);
void zipsize2(void);
void len_err(void);
void std_err(void);



#pragma pack(2)



struct local_head {
    unsigned short    ver;
    unsigned short    flag;
    unsigned short    method;
    unsigned int    timedate;
    unsigned int    crc;
    unsigned int    comp_size;
    unsigned int    uncomp_size;
    unsigned short    name_len;
    unsigned short    extra_len;
    /* filename */
    /* extra */
} local_head;

struct central_dir {
    unsigned short    ver_made;
    unsigned short    ver_ext;
    unsigned short    flag;
    unsigned short    method;
    unsigned int    timedate;
    unsigned int    crc;
    unsigned int    comp_size;
    unsigned int    uncomp_size;
    unsigned short    name_len;
    unsigned short    extra_len;
    unsigned short    comm_len;
    unsigned short    disk_start;
    unsigned short    int_attr;
    unsigned int    ext_attr;
    unsigned int    offset;
    /* filename */
    /* extra */
    /* comment */
} central_dir;



FILE    *fd;
int    minmax;
unsigned int    fsize;



int main(int argc, char *argv[]) {
    unsigned int    head_sign;
    int        err;


    setbuf(stdout, NULL);

    printf("\n"
        "ZIP fake resizer "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n\n");

    if(argc < 3) {
        printf("\nUsage: %s <0=min|1=max|2=random> <file.zip>\n", argv[0]);
        exit(1);
    }

    minmax = atoi(argv[1]);
    if((minmax > 2) || (minmax < 0)) {
        printf("\nError: 0 for 0 bytes, 1 for 4294967295 bytes and 2 for random\n");
        exit(1);
    }

    fd = fopen(argv[2], "r+b");
    if(!fd) std_err();

    srand(time(NULL));

    while(!feof(fd)) {
        err = fread(&head_sign, sizeof(head_sign), 1, fd);
        if(err != 1) len_err();

        switch(head_sign) {
            case PK1: zipsize1(); break;
            case PK2: zipsize2(); break;
            default: goto uscita; break;
        }
    }

    goto uscita;

/* exit */
uscita: {
    fclose(fd);
    printf("\n");
    return(0);
}

}



void zipsize1(void) {
    int    err;

    err = fread(&local_head, sizeof(struct local_head), 1, fd);
    if(err != 1) len_err();

    switch(minmax) {
        case 0: local_head.uncomp_size = 0x00000000; break;
        case 1: local_head.uncomp_size = 0xffffffff; break;
        case 2: local_head.uncomp_size = ((rand() + 1) * (rand() + 1)); break;
    }

    err = fseek(fd, - (sizeof(struct local_head)), SEEK_CUR);
    if(err < 0) std_err();
    err = fwrite(&local_head, sizeof(struct local_head), 1, fd);
    if(err != 1) std_err();
    fflush(fd);

    err = fseek(fd, local_head.comp_size + local_head.name_len + local_head.extra_len, SEEK_CUR);
    if(err < 0) std_err();
}



void zipsize2(void) {
    unsigned char    *buff;
    int        err;

    err = fread(&central_dir, sizeof(struct central_dir), 1, fd);
    if(err != 1) len_err();

    buff = alloca(central_dir.name_len + 1);
    if(!buff) std_err();

    err = fread(buff, central_dir.name_len, 1, fd);
    if(err != 1) len_err();
    buff[central_dir.name_len] = 0;

    printf("------------------------------------------------------------\n"
        "Filename: %s\n"
        "Original size:    Packed: %10lu    Unpacked: %10lu\n",
        buff,
        central_dir.comp_size,
        central_dir.uncomp_size);

    switch(minmax) {
        case 0: {
            central_dir.comp_size = 0x00000000;
            central_dir.uncomp_size = 0x00000000;
            }; break;
        case 1: {
            central_dir.comp_size = 0xffffffff;
            central_dir.uncomp_size = 0xffffffff;
            }; break;
        case 2: {
            central_dir.comp_size = ((rand() + 1) * (rand() + 1));
            central_dir.uncomp_size = ((rand() + 1) * (rand() + 1));
            }; break;
    }

    err = fseek(fd, - (sizeof(struct central_dir) + central_dir.name_len), SEEK_CUR);
    if(err < 0) std_err();
    err = fwrite(&central_dir, sizeof(struct central_dir), 1, fd);
    if(err != 1) std_err();
    fflush(fd);

    err = fseek(fd, central_dir.name_len + central_dir.extra_len + central_dir.comm_len, SEEK_CUR);
    if(err < 0) std_err();
}



void len_err(void) {
    printf("\nError: file is corrupted\n");
    exit(1);
}


void std_err(void) {
    perror("\nError");
    exit(1);
}

